import requests

from ....sdk.contracts.dtos.sdk.requests.world.delete import WorldDeleteRequest
from ..abstract import AbstractCommand


class WorldDeleteCommand(AbstractCommand):
    def execute(self, request: WorldDeleteRequest) -> None:
        response: requests.Response = requests.delete(
            url=f"http://{self.options.tld}/world/{request.id}", timeout=self.options.timeout
        )



import logging
import shutil
import uuid
from pathlib import Path

from pydantic import BaseModel

from ...sdk.contracts.dtos.island_lite import IslandLite
from ...sdk.contracts.dtos.sdk.wrapped_data import WrappedData
from ...sdk.contracts.errors.server.dao.conflict import DaoConflictError
from ...sdk.contracts.errors.server.dao.doesnotexist import DaoDoesNotExistError
from ...sdk.contracts.errors.server.dao.inconsistency import DaoInconsistencyError
from ...sdk.contracts.types.tile import TileType

logger = logging.getLogger()


class IslandDao(BaseModel):
    # ["base"]  / "worlds" / "world_id" / "islands" / "island_id" / metadata.json
    storage_base_path: Path

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.storage_base_path.mkdir(parents=True, exist_ok=True)

    ### Internal ##################################

    def _island_metadata_path(self, world_id: str, island_id: str) -> Path:
        # ["base"] / "worlds" / "world_id" / "islands" / "island_id" / metadata.json
        return self.storage_base_path / "worlds" / world_id / "islands" / island_id / "metadata.json"

    def generate(self, world_id: str, name: str | None, dimensions: tuple[int, int], biome: TileType) -> str:
        logger.debug("[IslandService] generating island skeleton")
        if name is None:
            name = "roshar"
        island: IslandLite = IslandLite(id=str(uuid.uuid4()), name=name, dimensions=dimensions, biome=biome)
        self.post(world_id=world_id, island=island)
        return island.id

    def get(self, world_id: str, island_id: str) -> WrappedData[IslandLite]:
        logger.debug("[IslandService] getting island metadata from storage")
        island_metadata_path: Path = self._island_metadata_path(world_id=world_id, island_id=island_id)
        if not island_metadata_path.exists():
            raise DaoDoesNotExistError("island metadata does not exist")
        with open(file=island_metadata_path.resolve().as_posix(), mode="r", encoding="utf-8") as file:
            json_data: str = file.read()
        return WrappedData[IslandLite].model_validate_json(json_data)

    def post(self, world_id: str, island: IslandLite) -> None:
        logger.debug("[IslandService] posting island metadata to storage")
        island_metadata_path: Path = self._island_metadata_path(world_id=world_id, island_id=island.id)
        if island_metadata_path.exists():
            raise DaoConflictError("island metadata already exists")
        if not island_metadata_path.parent.exists():
            logger.debug("[IslandService] island metadata folder creating ..")
            island_metadata_path.parent.mkdir(parents=True, exist_ok=True)

        nonce: str = str(uuid.uuid4())
        wrapped_data: WrappedData[IslandLite] = WrappedData[IslandLite](data=island, nonce=nonce)
        wrapped_data_raw: str = wrapped_data.model_dump_json(indent=4)
        with open(file=island_metadata_path.resolve().as_posix(), mode="w", encoding="utf-8") as file:
            file.write(wrapped_data_raw)

        # now validate we stored
        stored_island: WrappedData[IslandLite] = self.get(world_id=world_id, island_id=island.id)
        if stored_island.nonce != nonce:
            msg: str = f"storage inconsistency detected while storing island {island.id} - nonce mismatch!"
            raise DaoInconsistencyError(msg)

    def put(self, world_id: str, island: IslandLite) -> None:
        logger.debug("[IslandService] putting island metadata to storage")
        island_metadata_path: Path = self._island_metadata_path(world_id=world_id, island_id=island.id)
        if not island_metadata_path.parent.exists():
            logger.debug("[IslandService] island metadata folder creating ..")
            island_metadata_path.parent.mkdir(parents=True, exist_ok=True)

        nonce: str = str(uuid.uuid4())
        wrapped_data: WrappedData[IslandLite] = WrappedData[IslandLite](data=island, nonce=nonce)
        wrapped_data_raw: str = wrapped_data.model_dump_json(indent=4)
        with open(file=island_metadata_path.resolve().as_posix(), mode="w", encoding="utf-8") as file:
            file.write(wrapped_data_raw)

        # now validate we stored
        stored_island: WrappedData[IslandLite] = self.get(world_id=world_id, island_id=island.id)
        if stored_island.nonce != nonce:
            msg: str = f"storage inconsistency detected while storing island {island.id} - nonce mismatch!"
            raise DaoInconsistencyError(msg)

    def delete(self, world_id: str, island_id: str) -> None:
        logger.debug("[IslandService] deleting island data from storage")
        island_metadata_path: Path = self._island_metadata_path(world_id=world_id, island_id=island_id)
        if not island_metadata_path.exists():
            raise DaoDoesNotExistError("island metadata does not exist")
        # remove "island_id"/ and lower
        shutil.rmtree(island_metadata_path.parent.resolve().as_posix())



import logging
import shutil
import uuid
from pathlib import Path

from pydantic import BaseModel

from ...sdk.contracts.dtos.sdk.wrapped_data import WrappedData
from ...sdk.contracts.dtos.world_lite import WorldLite
from ...sdk.contracts.errors.server.dao.conflict import DaoConflictError
from ...sdk.contracts.errors.server.dao.doesnotexist import DaoDoesNotExistError
from ...sdk.contracts.errors.server.dao.inconsistency import DaoInconsistencyError

logger = logging.getLogger()


class WorldDao(BaseModel):
    # ["base"]  / "worlds" / "world_id" / metadata.json
    storage_base_path: Path

    def _world_metadata_path(self, world_id: str) -> Path:
        # ["base"] / "worlds" / "world_id" / metadata.json
        return self.storage_base_path / "worlds" / world_id / "metadata.json"

    ### Internal ##################################

    def generate(self, name: str | None) -> str:
        logger.debug("[WorldService] generating world skeleton")
        if name is None:
            name = "darkness"
        world: WorldLite = WorldLite(id=str(uuid.uuid4()), name=name)
        self.post(world=world)
        return world.id

    def get(self, world_id: str) -> WrappedData[WorldLite]:
        logger.debug("[WorldService] getting world metadata from storage")
        world_metadata_path: Path = self._world_metadata_path(world_id=world_id)
        if not world_metadata_path.exists():
            raise DaoDoesNotExistError("world metadata does not exist")
        with open(file=world_metadata_path.resolve().as_posix(), mode="r", encoding="utf-8") as file:
            json_data: str = file.read()
        return WrappedData[WorldLite].model_validate_json(json_data)

    def post(self, world: WorldLite) -> None:
        logger.debug("[WorldService] posting world metadata to storage")
        world_metadata_path: Path = self._world_metadata_path(world_id=world.id)
        if world_metadata_path.exists():
            raise DaoConflictError("world metadata already exists")
        if not world_metadata_path.parent.exists():
            logger.debug("[WorldService] world metadata folder creating ..")
            world_metadata_path.parent.mkdir(parents=True, exist_ok=True)

        nonce: str = str(uuid.uuid4())
        wrapped_data: WrappedData[WorldLite] = WrappedData[WorldLite](data=world, nonce=nonce)
        wrapped_data_raw: str = wrapped_data.model_dump_json(indent=4)
        with open(file=world_metadata_path.resolve().as_posix(), mode="w", encoding="utf-8") as file:
            file.write(wrapped_data_raw)

        # now validate we stored
        stored_world: WrappedData[WorldLite] = self.get(world_id=world.id)
        if stored_world.nonce != nonce:
            msg: str = f"storage inconsistency detected while storing world {world.id} - nonce mismatch!"
            raise DaoInconsistencyError(msg)

    def put(self, world: WorldLite) -> None:
        logger.debug("[WorldService] putting world metadata to storage")
        world_metadata_path: Path = self._world_metadata_path(world_id=world.id)
        if not world_metadata_path.parent.exists():
            logger.debug("[WorldService] world metadata folder creating ..")
            world_metadata_path.parent.mkdir(parents=True, exist_ok=True)

        nonce: str = str(uuid.uuid4())
        wrapped_data: WrappedData[WorldLite] = WrappedData[WorldLite](data=world, nonce=nonce)
        wrapped_data_raw: str = wrapped_data.model_dump_json(indent=4)
        with open(file=world_metadata_path.resolve().as_posix(), mode="w", encoding="utf-8") as file:
            file.write(wrapped_data_raw)

        # now validate we stored
        stored_world: WrappedData[WorldLite] = self.get(world_id=world.id)
        if stored_world.nonce != nonce:
            msg = f"storage inconsistency detected while storing world {world.id} - nonce mismatch!"
            raise DaoInconsistencyError(msg)

    def delete(self, world_id: str) -> None:
        logger.debug("[WorldService] deleting world data from storage")
        world_metadata_path: Path = self._world_metadata_path(world_id=world_id)
        if not world_metadata_path.exists():
            raise DaoDoesNotExistError("world metadata does not exist")
        # remove "world_id"/ and lower
        shutil.rmtree(world_metadata_path.parent.resolve().as_posix())



import logging

# import secrets
import uuid

from ....sdk.contracts.dtos.coordinate import Coordinate

# from ....sdk.contracts.dtos.island import Island
from ....sdk.contracts.dtos.island_lite import IslandLite
from ....sdk.contracts.dtos.tile import Tile
from ....sdk.contracts.dtos.window import Window
from ....sdk.contracts.dtos.world_lite import WorldLite
from ....sdk.contracts.types.connection import TileConnectionType
from ....sdk.contracts.types.tile import TileType
from ...dao.tile import TileDao
from .abstract import AbstractIslandFactory

logger = logging.getLogger()


class FlatIslandFactory(AbstractIslandFactory):
    tiledao: TileDao

    # @staticmethod
    # def tiles_process(island: Island, window: Window) -> None:
    #     range_x_min: int = window.min.x - 1
    #     range_x_max: int = window.max.x
    #     range_y_min: int = window.min.x - 1
    #     range_y_max: int = window.max.y
    #
    #     # Lets shovel in some biome default or dirt tiles!
    #     for x in range(range_x_min, range_x_max):
    #         for y in range(range_y_min, range_y_max):
    #             local_x = x + 1
    #             local_y = y + 1
    #             tile_id: str = f"tile_{local_x}_{local_y}"
    #             # print(tile_id)
    #
    #             # Mutate tiles to biome default (or dirt)
    #             FlatIslandFactory.mutate_tile(
    #                 island=island,
    #                 tile_id=tile_id,
    #                 mutate=90,  # percentage of 100%
    #                 type=(island.biome if island.biome else TileType.DIRT),
    #             )
    #
    #     # Add Eradicates (rocks)
    #     for x in range(range_x_min, range_x_max):
    #         for y in range(range_y_min, range_y_max):
    #             local_x = x + 1
    #             local_y = y + 1
    #             tile_id: str = f"tile_{local_x}_{local_y}"
    #             # print(tile_id)
    #             FlatIslandFactory.mutate_tile(island=island, tile_id=tile_id, mutate=0.5, type=TileType.ROCK)
    #
    #     # TODO: isolated ocean is NOT ocean, we MUST have path to the edge
    #     # Convert inner Ocean to Water Tiles
    #     for x in range(range_x_min, range_x_max):
    #         for y in range(range_y_min, range_y_max):
    #             local_x = x + 1
    #             local_y = y + 1
    #             tile_id: str = f"tile_{local_x}_{local_y}"
    #             # print(tile_id)
    #
    #             # Convert inner Ocean to Water Tiles
    #             FlatIslandFactory.brackish_tile(island=island, tile_id=tile_id)
    #
    #     # Erode Tiles (to make shore)
    #     for x in range(range_x_min, range_x_max):
    #         for y in range(range_y_min, range_y_max):
    #             local_x = x + 1
    #             local_y = y + 1
    #             tile_id: str = f"tile_{local_x}_{local_y}"
    #             # print(tile_id)
    #
    #             # Erode Tiles
    #             FlatIslandFactory.erode_tile(island=island, tile_id=tile_id)
    #
    #     # Grow Tiles
    #     for x in range(range_x_min, range_x_max):
    #         for y in range(range_y_min, range_y_max):
    #             local_x = x + 1
    #             local_y = y + 1
    #             tile_id: str = f"tile_{local_x}_{local_y}"
    #             # print(tile_id)
    #
    #             # Grow Tiles
    #             FlatIslandFactory.grow_tile(island=island, tile_id=tile_id)
    #
    # @staticmethod
    # def mutate_tile(island: Island, tile_id: str, mutate: float, type: TileType) -> None:
    #     if secrets.randbelow(100) <= mutate:
    #         # then we spawn the tile type
    #         island.tiles[tile_id].tile_type = type
    #
    # @staticmethod
    # def convert_tile(island: Island, tile_id: str, source: TileType, target: TileType) -> None:
    #     if tile_id in island.tiles and island.tiles[tile_id].tile_type == source:
    #         island.tiles[tile_id].tile_type = target
    #
    # @staticmethod
    # def adjecent_liquids(island: Island, tile_id: str) -> list[TileType]:
    #     return FlatIslandFactory.adjecent_to(island=island, tile_id=tile_id, types=[TileType.OCEAN, TileType.WATER])
    #
    # @staticmethod
    # def adjecent_to(island: Island, tile_id: str, types: list[TileType] | None) -> list[TileType]:
    #     adjecent_targets: list[TileType] = []
    #     for adjecent_dir, adjecent_id in island.tiles[tile_id].next.items():
    #         # assert adjecent_id == island.tiles[adjecent_id].id
    #         # print(f"tile {tile_id} -> {adjecent_dir} -> {adjecent_id}:{island.tiles[adjecent_id].id} -> {island.tiles[adjecent_id].tile_type}")
    #         if (
    #             island.tiles[adjecent_id].tile_type in types
    #             and island.tiles[adjecent_id].tile_type not in adjecent_targets
    #         ):
    #             adjecent_targets.append(island.tiles[adjecent_id].tile_type)
    #     # print(f"tile {tile_id} is adjecent to: {adjecent_targets}")
    #     return adjecent_targets
    #
    # @staticmethod
    # def grow_tile(island: Island, tile_id: str) -> None:
    #     # if island.tiles[tile_id].tile_type not in [TileType.UNKNOWN, TileType.OCEAN, TileType.WATER]:
    #
    #     # dirt -> grass
    #     if island.tiles[tile_id].tile_type == TileType.DIRT:
    #         adjecent_liquids: list[TileType] = FlatIslandFactory.adjecent_liquids(island=island, tile_id=tile_id)
    #         if TileType.WATER in adjecent_liquids and TileType.OCEAN not in adjecent_liquids:
    #             island.tiles[tile_id].tile_type = TileType.GRASS
    #
    #     # grass+water (no dirt/ocean) -> forest
    #     if island.tiles[tile_id].tile_type == TileType.GRASS:
    #         neighbors: list[TileType] = FlatIslandFactory.adjecent_to(
    #             island=island, tile_id=tile_id, types=[TileType.WATER, TileType.GRASS, TileType.OCEAN]
    #         )
    #
    #         # no forests grow by oceans
    #         if TileType.OCEAN in neighbors:
    #             return
    #
    #         if len(neighbors) > 1:
    #             # next to more than one kind (grass/water)
    #             island.tiles[tile_id].tile_type = TileType.FOREST
    #
    #     # # grass+(dirt)
    #     # if island.tiles[tile_id].tile_type == TileType.GRASS:
    #     #     neighbors: list[TileType] = FlatIslandFactory.adjecent_to(
    #     #         # TODO: review this josh ...
    #     #         island=island, tile_id=tile_id, types=[TileType.WATER, TileType.GRASS, TileType.DIRT, TileType.FOREST]
    #     #     )
    #
    # @staticmethod
    # def brackish_tile(island: Island, tile_id: str):
    #     # Convert inner Ocean to Water Tiles
    #
    #     # See if we are next to ocean
    #     neighbors: list[TileType] = FlatIslandFactory.adjecent_to(
    #         island=island, tile_id=tile_id, types=[TileType.OCEAN]
    #     )
    #     if len(neighbors) < 1:
    #         FlatIslandFactory.convert_tile(island=island, tile_id=tile_id, source=TileType.OCEAN, target=TileType.WATER)
    #
    # @staticmethod
    # def erode_tile(island: Island, tile_id: str) -> None:
    #     # shore erosion
    #     if island.tiles[tile_id].tile_type not in [TileType.UNKNOWN, TileType.OCEAN, TileType.WATER, TileType.SHORE]:
    #         adjecent_liquids: list[TileType] = FlatIslandFactory.adjecent_liquids(island=island, tile_id=tile_id)
    #         # Apply erosion - rocks and be left by oceans, everything else becomes shore
    #         if TileType.OCEAN in adjecent_liquids:
    #             if island.tiles[tile_id].tile_type not in (TileType.ROCK, TileType.SHORE):
    #                 previous_type = island.tiles[tile_id].tile_type
    #                 island.tiles[tile_id].tile_type = TileType.SHORE
    #                 # print(f"tile {tile_id}:{previous_type} converted to {TileType.SHORE}")

    def generate_ocean_block(self, world_id: str, island: IslandLite, window: Window):
        # 1. fill a blank nXm area with ocean
        range_x_min: int = window.min.x - 1
        range_x_max: int = window.max.x
        range_y_min: int = window.min.x - 1
        range_y_max: int = window.max.y
        for x in range(range_x_min, range_x_max):
            for y in range(range_y_min, range_y_max):
                local_x = x + 1
                local_y = y + 1

                tile_id: str = f"tile_{local_x}_{local_y}"
                local_tile: Tile = Tile(id=tile_id, tile_type=TileType.OCEAN)

                # store tile
                self.tiledao.post(world_id=world_id, island_id=island.id, tile=local_tile)
                logger.debug(f"({tile_id}) brought into existence as {TileType.OCEAN}")

                # Add tile_id to in-memory representation before storing
                island.tile_ids.add(tile_id)

                # store island update (tile addition)
                self.islanddao.put(world_id=world_id, island=island)

        # 2. connect the tiles (nXm)
        for x in range(range_x_min, range_x_max):
            for y in range(range_y_min, range_y_max):
                local_x = x + 1
                local_y = y + 1
                tile_id: str = f"tile_{local_x}_{local_y}"
                logger.debug(f"binding ({tile_id}) physically to peers")

                target_tile_id: str = f"tile_{local_x - 1}_{local_y}"
                if target_tile_id in island.tile_ids:
                    # load tile
                    local_tile: Tile = self.tiledao.get(
                        world_id=world_id, island_id=island.id, tile_id=target_tile_id
                    ).data

                    # update tile
                    local_tile.next[TileConnectionType.LEFT] = f"tile_{local_x - 1}_{local_y}"
                    logger.debug(
                        f"tile {tile_id} -> {TileConnectionType.LEFT} -> {local_tile.next[TileConnectionType.LEFT]}"
                    )

                    # store tile
                    self.tiledao.put(world_id=world_id, island_id=island.id, tile=local_tile)

                target_tile_id: str = f"tile_{local_x + 1}_{local_y}"
                if target_tile_id in island.tile_ids:
                    # load tile
                    local_tile: Tile = self.tiledao.get(
                        world_id=world_id, island_id=island.id, tile_id=target_tile_id
                    ).data

                    # update tile
                    local_tile.next[TileConnectionType.RIGHT] = f"tile_{local_x + 1}_{local_y}"

                    logger.debug(
                        f"tile {tile_id} -> {TileConnectionType.RIGHT} -> {local_tile.next[TileConnectionType.RIGHT]}"
                    )

                    # store tile
                    self.tiledao.put(world_id=world_id, island_id=island.id, tile=local_tile)

                target_tile_id: str = f"tile_{local_x}_{local_y - 1}"
                if target_tile_id in island.tile_ids:
                    # load tile
                    local_tile: Tile = self.tiledao.get(
                        world_id=world_id, island_id=island.id, tile_id=target_tile_id
                    ).data

                    # update tile
                    local_tile.next[TileConnectionType.UP] = f"tile_{local_x}_{local_y - 1}"
                    logger.debug(
                        f"tile {tile_id} -> {TileConnectionType.UP} -> {local_tile.next[TileConnectionType.UP]}"
                    )

                    # store tile
                    self.tiledao.put(world_id=world_id, island_id=island.id, tile=local_tile)

                target_tile_id: str = f"tile_{local_x}_{local_y + 1}"
                if target_tile_id in island.tile_ids:
                    # load tile
                    local_tile: Tile = self.tiledao.get(
                        world_id=world_id, island_id=island.id, tile_id=target_tile_id
                    ).data

                    # update tile
                    local_tile.next[TileConnectionType.DOWN] = f"tile_{local_x}_{local_y + 1}"
                    logger.debug(
                        f"tile {tile_id} -> {TileConnectionType.DOWN} -> {local_tile.next[TileConnectionType.DOWN]}"
                    )

                    # store tile
                    self.tiledao.put(world_id=world_id, island_id=island.id, tile=local_tile)

    def create(self, world_id: str, name: str | None, dimensions: tuple[int, int], biome: TileType) -> IslandLite:
        if name is None:
            name = "roshar"

        # 1. blank, named island
        island: IslandLite = IslandLite(id=str(uuid.uuid4()), name=name, dimensions=dimensions, biome=biome)
        self.islanddao.post(world_id=world_id, island=island)
        # island: Island = Island(id=str(uuid.uuid4()), name=name, dimensions=dimensions, biome=biome)

        # Define the maximum size
        max_x, max_y = dimensions

        # Generate an empty 2D block of ocean
        window: Window = Window(min=Coordinate(x=1, y=1), max=Coordinate(x=max_x, y=max_y))
        # print(window.model_dump_json())
        self.generate_ocean_block(world_id=world_id, island=island, window=window)

        # # TODO:
        # # Apply our terrain generation
        # window = Window(min=Coordinate(x=2, y=2), max=Coordinate(x=max_x - 1, y=max_y - 1))
        # # print(window.model_dump_json())
        # FlatIslandFactory.tiles_process(island=island, window=window)

        return island



import logging

from pydantic import BaseModel

from ...sdk.contracts.dtos.island import Island
from ...sdk.contracts.dtos.island_lite import IslandLite
from ...sdk.contracts.dtos.sdk.requests.island.create import IslandCreateRequest
from ...sdk.contracts.dtos.sdk.requests.island.delete import IslandDeleteRequest
from ...sdk.contracts.dtos.sdk.requests.island.get import IslandGetRequest
from ...sdk.contracts.dtos.sdk.requests.world.create import WorldCreateRequest
from ...sdk.contracts.dtos.sdk.requests.world.delete import WorldDeleteRequest
from ...sdk.contracts.dtos.sdk.requests.world.get import WorldGetRequest
from ...sdk.contracts.dtos.tile import Tile
from ...sdk.contracts.dtos.world import World
from ...sdk.contracts.dtos.world_lite import WorldLite
from ..dao.island import IslandDao
from ..dao.tile import TileDao
from ..dao.world import WorldDao
from ..factories.island.flat import FlatIslandFactory
from ..factories.world.world import WorldFactory

logger = logging.getLogger()


class StateService(BaseModel):
    worlddao: WorldDao
    islanddao: IslandDao
    tiledao: TileDao

    worldfactory: WorldFactory | None = None
    flatislandfactory: FlatIslandFactory | None = None

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.worldfactory = WorldFactory(worlddao=self.worlddao)
        self.flatislandfactory = FlatIslandFactory(islanddao=self.islanddao, tiledao=self.tiledao)

    ### World ##################################

    def world_create(self, request: WorldCreateRequest) -> str:
        logger.debug("[StateService] creating world")
        return self.worldfactory.create(name=request.name)

    def world_lite_get(self, request: WorldGetRequest) -> WorldLite:
        return self.worlddao.get(world_id=request.id).data

    def world_get(self, request: WorldGetRequest) -> World:
        # logger.info("- 1 -----------------------------------------")

        # Build a complete World from Lite objects
        world_lite: WorldLite = self.worlddao.get(world_id=request.id).data
        island_ids: set[str] = world_lite.island_ids
        # logger.info(f"island_ids={island_ids}")
        partial_world = world_lite.model_dump(exclude={"island_ids"})
        # logger.info("- 2 -----------------------------------------")
        # logger.info(partial_world)
        world: World = World.model_validate(partial_world)
        # logger.info("- 3 -----------------------------------------")
        # logger.info(world)
        # logger.info("- 4 -----------------------------------------")
        for island_id in island_ids:
            local_island: Island = self.island_get(request=IslandGetRequest(world_id=request.id, island_id=island_id))
            world.islands[island_id] = local_island
        return world

    def world_delete(self, request: WorldDeleteRequest) -> None:
        logger.debug("[StateService] deleting world")
        self.worlddao.delete(world_id=request.id)

    ### Island ##################################

    def island_create(self, request: IslandCreateRequest) -> str:
        logger.debug("[StateService] creating island")
        new_island: IslandLite = self.flatislandfactory.create(
            world_id=request.world_id, name=request.name, dimensions=request.dimensions, biome=request.biome
        )
        # add new island to world
        # add island to world and store
        world_lite: WorldLite = self.worlddao.get(world_id=request.world_id).data
        world_lite.island_ids.add(new_island.id)
        self.worlddao.put(world=world_lite)

        return new_island.id

    def island_delete(self, request: IslandDeleteRequest) -> None:
        msg: str = f"[WorldService] deleting island {id}"
        logger.debug(msg)
        self.islanddao.delete(world_id=request.world_id, island_id=request.island_id)

    def island_lite_get(self, request: IslandGetRequest) -> IslandLite:
        return self.islanddao.get(world_id=request.world_id, island_id=request.island_id).data

    def island_get(self, request: IslandGetRequest) -> Island:
        # Builds a complete Island from Lite objects
        island_lite: IslandLite = self.islanddao.get(world_id=request.world_id, island_id=request.island_id).data
        tile_ids: set[str] = island_lite.tile_ids
        island_partial = island_lite.model_dump(exclude={"tile_ids"})
        island: Island = Island.model_validate(island_partial)

        # re-hydrate the tiles
        for tile_id in tile_ids:
            tile: Tile = self.tiledao.get(world_id=request.world_id, island_id=island.id, tile_id=tile_id).data
            island.tiles[tile_id] = tile

        return island



from pydantic import BaseModel

from ..types.tile import TileType


class IslandLite(BaseModel):
    id: str
    name: str
    rbac: dict = {}
    tile_ids: set[str] = set()
    dimensions: tuple[int, int] | None = None
    biome: TileType | None = None



from pydantic import BaseModel

from ..dtos.entity import Entity
from ..types.connection import TileConnectionType
from ..types.tile import TileType


class Tile(BaseModel):
    id: str
    tile_type: TileType
    rbac: dict = {}
    next: dict[TileConnectionType, str] = {}  # str=id
    contains: dict[str, Entity] = {}  # str=id



import logging
import os
import uuid
from pathlib import Path

from pydantic import BaseModel

from ...sdk.contracts.dtos.sdk.wrapped_data import WrappedData
from ...sdk.contracts.dtos.tile import Tile
from ...sdk.contracts.errors.server.dao.conflict import DaoConflictError
from ...sdk.contracts.errors.server.dao.doesnotexist import DaoDoesNotExistError
from ...sdk.contracts.errors.server.dao.inconsistency import DaoInconsistencyError
from ...sdk.contracts.types.tile import TileType

logger = logging.getLogger()


class TileDao(BaseModel):
    # ["base"] / "worlds" / "wold_id" / "islands" / "island_id" / "tiles" / "tile_id.json"
    storage_base_path: Path

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.storage_base_path.mkdir(parents=True, exist_ok=True)

    ### Internal ##################################

    def _tile_path(self, world_id: str, island_id: str, tile_id: str) -> Path:
        # ["base"] / "worlds" / "wold_id" / "islands" / "island_id" / "tiles" / "tile_id.json"
        return self.storage_base_path / "worlds" / world_id / "islands" / island_id / "tiles" / f"{tile_id}.json"

    def generate(self, world_id: str, island_id: str, tile_type: TileType = TileType.UNKNOWN) -> str:
        logger.debug("[TileService] generating tile skeleton")
        tile: Tile = Tile(id=str(uuid.uuid4()), tile_type=tile_type)
        self.post(world_id=world_id, island_id=island_id, tile=tile)
        return tile.id

    def get(self, world_id: str, island_id: str, tile_id: str) -> WrappedData[Tile]:
        logger.debug("[TileService] getting island data from storage")
        tile_metadata_path: Path = self._tile_path(world_id=world_id, island_id=island_id, tile_id=tile_id)
        if not tile_metadata_path.exists():
            raise DaoDoesNotExistError("tile metadata does not exist")
        with open(file=tile_metadata_path.resolve().as_posix(), mode="r", encoding="utf-8") as file:
            json_data: str = file.read()
        return WrappedData[Tile].model_validate_json(json_data)

    def post(self, world_id: str, island_id: str, tile: Tile) -> None:
        logger.debug("[TileService] posting tile data to storage")
        tile_metadata_path: Path = self._tile_path(world_id=world_id, island_id=island_id, tile_id=tile.id)
        if tile_metadata_path.exists():
            raise DaoConflictError("tile metadata already exists")
        if not tile_metadata_path.parent.exists():
            logger.debug("[TileService] tile metadata folder creating ..")
            tile_metadata_path.parent.mkdir(parents=True, exist_ok=True)
        nonce: str = str(uuid.uuid4())
        wrapped_data: WrappedData[Tile] = WrappedData[Tile](data=tile, nonce=nonce)
        wrapped_data_raw: str = wrapped_data.model_dump_json(indent=4)
        with open(file=tile_metadata_path.resolve().as_posix(), mode="w", encoding="utf-8") as file:
            file.write(wrapped_data_raw)

        # now validate we stored
        stored_tile: WrappedData[Tile] = self.get(world_id=world_id, island_id=island_id, tile_id=tile.id)
        if stored_tile.nonce != nonce:
            msg: str = f"storage inconsistency detected while storing tile {tile.id} - nonce mismatch!"
            raise DaoInconsistencyError(msg)

    def put(self, world_id: str, island_id: str, tile: Tile) -> None:
        logger.debug("[TileService] putting tile data to storage")
        tile_metadata_path: Path = self._tile_path(world_id=world_id, island_id=island_id, tile_id=tile.id)
        if not tile_metadata_path.parent.exists():
            logger.debug("[TileService] tile metadata folder creating ..")
            tile_metadata_path.parent.mkdir(parents=True, exist_ok=True)
        nonce: str = str(uuid.uuid4())
        wrapped_data: WrappedData[Tile] = WrappedData[Tile](data=tile, nonce=nonce)
        wrapped_data_raw: str = wrapped_data.model_dump_json(indent=4)
        with open(file=tile_metadata_path.resolve().as_posix(), mode="w", encoding="utf-8") as file:
            file.write(wrapped_data_raw)

        # now validate we stored
        stored_tile: WrappedData[Tile] = self.get(world_id=world_id, island_id=island_id, tile_id=tile.id)
        if stored_tile.nonce != nonce:
            msg: str = f"storage inconsistency detected while storing tile {tile.id} - nonce mismatch!"
            raise DaoInconsistencyError(msg)

    def delete(self, world_id: str, island_id: str, tile_id: str) -> None:
        logger.debug("[TileService] deleting tile data from storage")
        tile_metadata_path: Path = self._tile_path(world_id=world_id, island_id=island_id, tile_id=tile_id)
        if not tile_metadata_path.exists():
            raise DaoDoesNotExistError("tile metadata does not exist")
        os.remove(path=tile_metadata_path.resolve().as_posix())
